#include <chrono>
#include <iomanip>
#include <iostream>

using namespace std;
using namespace std::chrono;

double piIntStepTrapezoidalRule(int i, const double& step)	{
	double x = (i+0.5)*step;
	return 1.0/(1.0+x*x);
}

int main()	{
	unsigned long nsteps = 0u;
	cout<<"Number of integration steps: ";
	cin>>nsteps;
	if (nsteps <= 0)	{
		cerr<<"Invalid input!"<<endl;
		return 1;
	}

	high_resolution_clock::time_point start = high_resolution_clock::now();

	double sum = 0.0, step = 1.0/nsteps;
	for (unsigned long i = 0; i < nsteps; ++i)
		sum += piIntStepTrapezoidalRule(i, step);

	double pi = 4.0*step*sum;

	double runtime = duration_cast<duration<double>>(high_resolution_clock::now()-start).count();

	cout<<"pi: "<<setprecision(15)<<pi<<endl;
	cout<<"run time: "<<runtime<<" s ("<<(1e9*runtime)/nsteps<<" ns per iteration)"<<endl;

	return 0;
}

